-------------------------------------------------------
-- Design Name : mb_hv_ctrl_tb_01
-- File Name   : mb_hv_ctrl_tb_01.vhd
-- Function    : mDOM mainboard HV control test bench
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2020-11-22
-------------------------------------------------------
library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
 
entity mb_hv_ctrl_tb_01 is
  generic( REG_WIDTH : integer := 32);
end mb_hv_ctrl_tb_01;

architecture behavior of mb_hv_ctrl_tb_01 is
 
  component mb_hv_ctrl_top is 
   port(    

      CPLD_MCU_AFE_RST   : in  std_logic;  -- connected to MCU port PA10, register reset
      CPLD_MCU_CAL_SS    : in  std_logic;  -- connected to MCU port  PE4, low active frame signal (like nCS)
      CPLD_MCU_AFE_SEL   : in  std_logic;  -- connected to MCU port  PA9, to distinguish between AFE and mDAB access
      CPLD_MCU_CAL_SCLK  : in  std_logic;  -- connected to MCU port  PE2, SPI clock
      CPLD_MCU_CAL_MOSI  : in  std_logic;  -- connected to MCU port  PE6, SPI data by MCU
		
      CPLD_MCU_USART6_RX : out std_logic;  -- connected to MCU port PG9, might also be used as CPLD_MCU_AFE_MISO(?)
      CPLD_MCU_USART6_TX : in  std_logic;  -- connected to MCU port PG14
                                       
      HV_UART_RX         : in  std_logic_vector(23 downto 0);   -- received from to PMT base UART
		HV_UART_TX         : out std_logic_vector(23 downto 0);   -- to be sent to PMT base UART 
		
		AFE_EN             : out std_logic_vector(23 downto 0)    -- analog frontend power on / off 
		
     );
   end component mb_hv_ctrl_top ;
 
 component mb_hv_mcu_signals_top is 
  port(    
      clk                : in  std_logic;  --
		mcu_data           : in  std_logic_vector (28 downto 0); -- data to be sent
		done               : out std_logic;  -- SPI cycle done
      CPLD_MCU_CAL_SS    : out std_logic;  -- connected to MCU port  PE4, low active frame signal (like nCS)
      CPLD_MCU_AFE_SEL   : in  std_logic;  -- connected to MCU port  PA9, to distinguish between AFE and mDAB access
      CPLD_MCU_CAL_SCLK  : out std_logic;  -- connected to MCU port  PE2, SPI clock
      CPLD_MCU_CAL_MOSI  : out std_logic;  -- connected to MCU port  PE6, SPI data
      CPLD_MCU_USART6_TX : out std_logic;  -- connected to MCU port PG14
		HV_UART_RX         : out std_logic_vector(23 downto 0)   --	
     );
  end component mb_hv_mcu_signals_top ;
 

     signal CPLD_MCU_AFE_RST   : std_logic := '1';  
     signal CPLD_MCU_CAL_SS    : std_logic := '1';  
     signal CPLD_MCU_AFE_SEL   : std_logic ;   
     signal CPLD_MCU_CAL_SCLK  : std_logic ;   
     signal CPLD_MCU_CAL_MOSI  : std_logic ;   
 
     signal CPLD_MCU_USART6_RX : std_logic ;  
     signal CPLD_MCU_USART6_TX : std_logic ;  
  
     signal HV_UART_RX         : std_logic_vector(23 downto 0);
	  signal HV_UART_TX         : std_logic_vector(23 downto 0);
	  
	  signal AFE_EN             : std_logic_vector(23 downto 0); 
	  
     signal clk                : std_logic :='0'; -- to run the data generation module
     signal mcu_data           : std_logic_vector (28 downto 0); -- data to be sent
     signal done               : std_logic := '0';  -- SPI cycle done

     --signal reset              : std_logic    := 'U';
     constant clk_period       : time := 1us;
  
begin



  clk <= not clk after clk_period / 2;
 
 -- CPLD_MCU_AFE_RST <= '1', '0' after 500 ns; --, '0' after 50 ns;
 --  RS232_RXD   <= tx_out;     -- not tx_out;   external UART connected for serilization / deserialization
--  rx_in       <= RS232_TXD;  -- "not" due to the transceiver LTC1386 inverters

-- DISCR_OUT_gen: process
--   variable i   : integer;
--  begin
--    while (true) loop
--      for i in 0 to 7 loop
--       DISCR_OUT_P(i) <= '1';
--       DISCR_OUT_N(i) <= '0';
--      end loop;
--       wait for 2 ns;
--      for i in 0 to 7 loop
--       DISCR_OUT_P(i) <= '0';
--       DISCR_OUT_N(i) <= '1';
--      end loop;
--      wait for 8 ns;
--    end loop;
--  end process DISCR_OUT_gen;  
	
 SPI_test: process
  	
--	 variable message     : string (1 to message_len) := "  DAC  7  0 5 1234  " & cr & lf;
--  	constant DAC_CMD_STRG : string := "dac75123  " &cr; --&lf&"ver  " &cr&lf&cr&lf;
--  	variable i           : integer;

	begin
	
	   CPLD_MCU_AFE_SEL   <= '0';
		CPLD_MCU_AFE_RST   <= '1';
 
      wait for clk_period * 10;
		CPLD_MCU_AFE_RST   <= '0';
		wait for clk_period * 10;
		CPLD_MCU_AFE_SEL   <= '1'; -- start MCU signal generation

 -- mcu_data format
 -- bit_31..29	reserved
 -- bit_28..24 AFE_UART_SEL_4..0
 -- bit_23..0	AFE_EN_23..0
     
   MCU_SIGNAL_GEN: for i in 0 to 3 loop
			
		 if    (i=0) then mcu_data  <= B"01000" & X"234567";  
       elsif (i=1) then mcu_data  <= B"00001" & X"123456";
       elsif (i=2) then mcu_data  <= B"00010" & X"89abcd";
       elsif (i=3) then mcu_data  <= B"00011" & X"fffabc";
       end if;
			
		 wait until (done = '1');
		 wait until (done = '0');
		 wait for clk_period * 30;
		 CPLD_MCU_AFE_SEL   <= '1'; -- start MCU signal generation
		 --wait for clk_period * 30;
			--CPLD_MCU_AFE_SEL   <= '0'; -- reset MCU signal generation
 	    wait for clk_period *10; --
			
            
    end loop MCU_SIGNAL_GEN;
	    
   wait for clk_period * 10;
   
	wait;
  	    
	end process SPI_test; 

--   mr_spi_miso <= sl_spi_miso after 10 ns;
--   sl_spi_sync <= mr_spi_sync after 10 ns;
--   sl_spi_sclk <= mr_spi_sclk after 10 ns;
--   sl_spi_mosi <= mr_spi_mosi after 10 ns; 
	    
-- instantiate the units under test (uut)

--  DTB_SPI_CTRL: component DTB_spi_ctrl_8bit_01_top port map
--   (
--    mr_reset     => reset    ,
--    mr_clk       => mr_clk      ,
--    mr_tx_addr   => mr_tx_addr ,
--    mr_tx_data   => mr_tx_data  ,
--    mr_rx_data   => mr_rx_data  ,
--    mr_spi_run   => mr_spi_run  ,
--    mr_spi_wr    => mr_spi_wr   ,
--    mr_spi_done  => mr_spi_done ,
--    mr_spi_sync  => mr_spi_sync ,
--    mr_spi_sclk  => mr_spi_sclk ,
--    mr_spi_mosi  => mr_spi_mosi ,
--    mr_spi_miso  => mr_spi_miso ,
--    
--   );
 
 mb_hv_mcu_signals_top_port_map: component mb_hv_mcu_signals_top port map
   (    
      clk                => clk              ,  --
		mcu_data           => mcu_data         , -- data to be sent
		done               => done             ,  -- SPI cycle done
      CPLD_MCU_CAL_SS    => CPLD_MCU_CAL_SS  ,  -- connected to MCU port  PE4, low active frame signal (like nCS)
      CPLD_MCU_AFE_SEL   => CPLD_MCU_AFE_SEL ,  -- connected to MCU port  PA9, to distinguish between AFE and mDAB access
      CPLD_MCU_CAL_SCLK  => CPLD_MCU_CAL_SCLK,  -- connected to MCU port  PE2, SPI clock
      CPLD_MCU_CAL_MOSI  => CPLD_MCU_CAL_MOSI,  -- connected to MCU port  PE6, SPI data
      CPLD_MCU_USART6_TX => CPLD_MCU_USART6_TX,  -- connected to MCU port PG14
      HV_UART_RX		    => HV_UART_RX
     );
  
 
 
  mb_hv_ctrl_top_PORT_MAP: component mb_hv_ctrl_top port map 
    (    
      CPLD_MCU_AFE_RST   => CPLD_MCU_AFE_RST , 
      CPLD_MCU_CAL_SS    => CPLD_MCU_CAL_SS  , 
      CPLD_MCU_AFE_SEL   => CPLD_MCU_AFE_SEL , 
      CPLD_MCU_CAL_SCLK  => CPLD_MCU_CAL_SCLK, 
      CPLD_MCU_CAL_MOSI  => CPLD_MCU_CAL_MOSI, 
      CPLD_MCU_USART6_RX => CPLD_MCU_USART6_RX,  
      CPLD_MCU_USART6_TX => CPLD_MCU_USART6_TX,                                      
      HV_UART_RX         => HV_UART_RX,
		HV_UART_TX         => HV_UART_TX,	                     
		AFE_EN             => AFE_EN     	
     );
 
end architecture behavior;
